@@ -18,6 +18,8 @@ Copyright 2011 Mozes, Inc.
 """
 from copy import deepcopy
 
+from stomper import Frame
+
 reservedHeaders = ['message-id', 'timestamp', 'expires', 'priority', 'destination']
 
 def filterReservedHeaders(headers):
@@ -33,4 +35,9 @@ def cloneStompMessageForErrorDest(msg):
     errMsg['headers']['persistent'] = 'true'
     return errMsg
 
-    
\ No newline at end of file
+def createFrame(message):
+    frame = Frame()
+    frame.cmd = message['cmd']
+    frame.headers = message['headers']
+    frame.body = message['body']
+    return frame
